-- Upgrade routine from official to ~Snib versions
-- pass optional @forceUpgrade to run this even if the version is up to date
function PR.UpgradeVersion(forceUpgrade)
    local snibTag = PR.Config.Version:find("~Snib")
    local configVersionNumber
    local versionNumber

    if snibTag then
        configVersionNumber = PR.Config.Version:sub(1, snibTag - 1)
        versionNumber = PR.Version:sub(1, PR.Version:find("~Snib") - 1)
        -- simpple version number string comparison
        -- if we ever use double digit version numbers we need to split it and
        -- compare each part
        if configVersionNumber > versionNumber then
            -- plugin downgrade detected, don't touch the database
            PR.Print(PR.Color.Red .. "WARNING: Prospector plugin is of a lower" ..
                " version than its database, please upgrade to the latest version" ..
                PR.Color.TERMINATOR, true)
            return
        end
    end

    PR.Print("/// Prospector is being upgraded to version " .. PR.Version, true)
    if not snibTag or forceUpgrade then
        -- re-create all system ore summaries (this may take a few seconds!)
        PR.Print("Prospector database upgrade in progress...", true)
        PR.UpgradeSectors()
        PR.RecreateAllSystemData()
        PR.Print("Prospector database upgrade completed.", true)

        -- convert legacy config values
        if PR.Config.Quiet then
            if PR.Config.Quiet == "YES" then PR.Config.VerbosityLevel = 1
            else PR.Config.VerbosityLevel = 3
            end
        end
        if PR.Config.COLOR and type(PR.Config.COLOR) ~= "boolean" then
            if PR.Config.COLOR == "WHITE" then PR.Config.COLOR = false
            elseif PR.Config.COLOR == "COLOR" then PR.Config.COLOR = true
            end
        end
        if PR.Config.Platform then
            if PR.Config.Platform == "PC" then
                PR.Config.PrintToHUD = true
            else
                PR.Config.PrintToHUD = false
            end
        end
        -- remove now redundant config values
        PR.Config.Audit = nil
        PR.Config.AuditSysNext = nil
        PR.Config.BestRoid = nil
        PR.Config.BestValue = nil
        PR.Config.DisplaySort = nil
        PR.Config.Platform = nil
        PR.Config.Quiet = nil
        PR.Config.SRR = nil
        PR.Config.SRT = nil
        PR.Config.SRU = nil
    else
        -- do version upgrade here in the future
    end

    PR.Config.Version = PR.Version
    PR.Print("Prospector configuration has been updated.", true)
    PR.SaveConfigNow()
    PR.Print("/// Prospector upgrade to version " .. PR.Version ..
        " is finished!", true)
end

-- run forced database upgrade via user command
function PR.ForceUpgradeVersion()
    PR.UpgradeVersion(true)
end

-- compacts all sector data from legacy databases and fixes maxId
function PR.UpgradeSectors()
    for systemId = 1, PR.SystemOwnership.n do
        for sectorId = systemId * 256 - 255, systemId * 256 do
            local sectorData = PR.LoadSectorData(sectorId)
            if sectorData and type(sectorData) == "table" and next(sectorData) ~= nil then
                if not sectorData.RoidList or not next(sectorData.RoidList) then
                    -- reset sector if we have no known asteroids in it
                    sectorData = {}
                else
                    local badSectors = {}
                    local badRoids = {}
                    local maxId = 0
                    for roidId, roidData in pairs(sectorData.RoidList) do
                        -- check for invalid asteroid data, from when official
                        -- version would save to the wrong asteroid or update
                        -- it incorrectly - all we can detect here are
                        -- asteroids with over 100% ore contents
                        local total = 0.0
                        for i = 1, PR.Ore.n do
                            total = total + (roidData[PR.Ore[i].shortname] or 0)
                        end
                        if total >= 100.2 then
                            -- bad data, delete this asteroid from db once
                            -- we're done iterating over this table
                            table.insert(badRoids, roidId)
                            if not badSectors[sectorId] then
                                -- only warn once per sector
                                PR.Print("Deleting some invalid asteroid data in " ..
                                    ShortLocationStr(sectorId), true)
                                badSectors[sectorId] = true
                            end
                        else
                            -- get max id from actually scanned asteroids
                            if roidId > maxId then maxId = roidId end
                            -- remove 0 mass
                            if sectorData.RoidList[roidId].Pt and
                                sectorData.RoidList[roidId].Pt == 0 then
                                sectorData.RoidList[roidId].Pt = nil
                            end
                            -- remove legacy data
                            sectorData.RoidList[roidId].Id = nil
                            sectorData.RoidList[roidId].Src = nil
                            sectorData.RoidList[roidId].Sid = nil
                            sectorData.RoidList[roidId].XPV = nil
                        end
                    end
                    -- delete bad roids now
                    for i = 1, #badRoids do
                        sectorData.RoidList[ badRoids[i] ] = nil
                    end
                    if maxId > 0 then
                        -- update maxId
                        sectorData.MaxId = maxId
                    else
                        sectorData = {}
                    end
                end
                -- remove more legacy data
                sectorData.Bots = nil
                sectorData.RoidStatus = nil
                sectorData.SectorOres = nil
                sectorData.Unscanned = nil
                sectorData.Usable = nil
                PR.SaveSectorData(sectorId, sectorData)
            end
        end
    end
end
